function [chi2,w,TransImg1,TransImg2,TransDImg,TransW,deltaD,bandsize] = ISFA(img1, p1, img2, p2, trhd, norm_flag, sqrt_flag)
% Iterative Slow Feature Analysis Algorithm for Change Detection
% 
% Input:
% ISFA(img1, p1, img2, p2, norm_flag, sqrt_flag):
%   img1, img2: input data matrix for transformation, size of nband*pixel_num
%   p1, p2: size of input data matrix, [ncol, nrow, nband]
%   norm_flag: if true, the transformation matrix will be normlized
%   sqrt_flag: if true, the eigenvalue will be updated by sqrt
% 
% Output:
% [chi2,w,TransImg1,TransImg2,TransDImg,TransW,deltaD,bandsize]:
%   chi2: chi2 distance
%   w: weights following chi2 distribution
%   TransImg1, TransImg2: transformed features of img 1 and 2
%   TransDImg: diffrence feature
%   TransW: transformation matrix
%   deltaD: eigenvalue during iterations
%   bandsize: output band size
% 

if nargin < 5
    error('Function needs at least 5 params')
elseif nargin == 5
    norm_flag = true;
    sqrt_flag = false;
elseif nargin == 6
    sqrt_flag = false;
elseif nargin > 7
    error('Too many params')
end
    
% Testing image size
if all(p1==p2)==0
    error('Input image with difference sizes');
end

nband=p1(3);

% Image size
[bandsize,~]=size(img1);
bandnum=bandsize;
imgnum=p1(1)*p1(2);
X=img1;
Y=img2;

% Weight initialization
w=ones(1,imgnum);
% w=initw;
delta=[];
delta0=zeros(bandnum,1);
deltaw=[];
deltaw0=zeros(bandnum,1);
deltaD=[];
deltaD0=zeros(bandnum,1);
% Iterations
iteration=100;
% Iteration
for i=1:iteration
%     Weights
    W=repmat(w,bandnum,1);
    sumw=sum(w);
%     Normalization with weights: zero means
    meanX= sum(X.*W,2)/sumw;
    meanY= sum(Y.*W,2)/sumw;
%     Normalization with weights: unit variances
    varX=sum(W.*(X-repmat(meanX,1,imgnum)).^2,2)/((imgnum-1)*sumw/imgnum);
    varY=sum(W.*(Y-repmat(meanY,1,imgnum)).^2,2)/((imgnum-1)*sumw/imgnum);
    sdX=sqrt(varX);
    sdY=sqrt(varY);
    Xc=(X-repmat(meanX,1,imgnum))./repmat(sdX,1,imgnum);
    Yc=(Y-repmat(meanY,1,imgnum))./repmat(sdY,1,imgnum); 
%     Slow feature analysis with weights
    A=(W.*(Xc-Yc))*(Xc-Yc)'/((imgnum-1)*sumw/imgnum);
    B=((W.*Xc)*Xc'+(W.*Yc)*Yc')/2/((imgnum-1)*sumw/imgnum);
%     Slow feature analysis with weights
    [V,D]=eig(A,B);
    [SD,index]=sort(diag(D));
    TransW=V(:,index);
    if norm_flag == true
        aux1 = TransW'*B*TransW;
        aux2 = 1./sqrt(diag(aux1));
        aux3 = repmat(aux2',bandnum,1);
        TransW = TransW.*aux3;
    end
    if sqrt_flag == true
        SD=sqrt(SD);
    end
%     Weight calculation
    SFA=TransW'*Xc-TransW'*Yc;
    
    T=sum(SFA.^2./repmat(SD,1,imgnum));
    deltaD=[deltaD,SD];
    
%     Drawing iteration
    plot(deltaD(:,:)')
    drawnow;
    
%     Convergence   
    disp(deltaD(:,end)');
    if abs(max(deltaD(:,end)-deltaD0))<trhd
        break;
    end
    deltaD0=deltaD(:,end);
    
%     Chi2 distribution
    w=1-chi2cdf(T,bandnum);
end
% End of iteration
% Output
chi2=T;
TransImg1=TransW'*Xc;
TransImg2=TransW'*Yc;
TransDImg=TransImg1-TransImg2;

end
